/*	libgemwidget - Gtk+ Embedded Widget library
 *
 *	Authors: YE Nan <nan.ye@orange-ftgourp.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */

#include <libintl.h>
#include <ctype.h>
#include <string.h>
#include <time.h>

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

#include <gtkautoscrolledwindow.h>

#include "gemfilechooser.h"
#include "gemresource.h"
#include "gemenv.h"

#include <libgnomevfs/gnome-vfs.h>
#include <libgnomevfs/gnome-vfs-mime.h>
#include <libgnomevfs/gnome-vfs-mime-handlers.h>
#include <libgnomevfs/gnome-vfs-mime-utils.h>
#include <libgnomevfs/gnome-vfs-mime-info.h>
#include <libgnomevfs/gnome-vfs-module-callback.h>
#include <libgnomevfs/gnome-vfs-standard-callbacks.h>
#include <libgnomevfs/gnome-vfs-xfer.h>
#include <libgnomevfs/gnome-vfs-types.h>
#include <libgnomevfs/gnome-vfs-application-registry.h>

#define DBGING 1
#define DBG(x...) do { if(DBGING) { \
                           g_print("%s(%d):", __FUNCTION__, __LINE__); \
                           g_print(x);    \
                           g_print("\n"); \
                      } \
                      }while(0)

#define _(x) gettext(x)

typedef	enum {
	GEM_FC_COL_CHECK,
	GEM_FC_COL_PIC,
	GEM_FC_COL_NAME,
	GEM_FC_COL_IS_FOLDER,
	GEM_FC_NUMS_COLS,
} GemFileChooserCols;

/* GObject, GtkObject methods
 */
static void gem_file_chooser_class_init			(GemFileChooserClass	*klass);
static void gem_file_chooser_init						(GemFileChooser				*window);
static void	gem_file_chooser_dispose				(GObject							*object);
static void	gem_file_chooser_finalize				(GObject							*object);

/* Default signal handlers
 */
static void gem_file_chooser_file_selected	(GemFileChooser				*filechooser);

static GemTreeView * parent_class = NULL;

enum {
  FILE_SELECTED,
  LAST_SIGNAL
};

static guint file_chooser_signals[LAST_SIGNAL];
/*
void
directory_changed ( GnomeVFSMonitorHandle * handle,
   	                const gchar * monitor_uri,
  	                const gchar * info_uri,
  	                GnomeVFSMonitorEventType event_type, gpointer user_data )
{
  if (!refresh_scheduled)
  {
    g_timeout_add (2000, (GSourceFunc) do_scheduled_refresh, NULL);
    refresh_scheduled = TRUE;
   }
}

static gboolean
do_scheduled_refresh (void)
{
 
  refresh_scheduled = FALSE;
  return FALSE;
}
*/
GType
gem_file_chooser_get_type (void)
{
	static GType file_chooser_type = 0;

  if (!file_chooser_type)
	{
		static const GTypeInfo file_chooser_info =
		{
			sizeof (GemFileChooserClass),
			NULL,		/* base_init */
			NULL,		/* base_finalize */
			(GClassInitFunc) gem_file_chooser_class_init,
			NULL,		/* class_finalize */
			NULL,		/* class_data */
			sizeof (GemFileChooser),
			0,			/* n_preallocs */
			(GInstanceInitFunc) gem_file_chooser_init,
		};

		file_chooser_type = g_type_register_static(GEM_TYPE_TREE_VIEW,
																						  "GemFileChooser",
																						  &file_chooser_info,
																						  0);
	}

	return file_chooser_type;
}

static void
gem_file_chooser_class_init (GemFileChooserClass * klass)
{
  GObjectClass	*object_class;

  object_class = G_OBJECT_CLASS(klass);
  parent_class = g_type_class_peek_parent(klass);

  object_class->dispose		= gem_file_chooser_dispose;
  object_class->finalize	= gem_file_chooser_finalize;

	klass->file_selected = gem_file_chooser_file_selected;

	/* Action signals */
  file_chooser_signals[FILE_SELECTED] = 
	g_signal_new("file_selected",
							 G_OBJECT_CLASS_TYPE(klass),
							 G_SIGNAL_RUN_LAST,
							 G_STRUCT_OFFSET(GemFileChooserClass, file_selected),
							 NULL, NULL,
							 gtk_marshal_VOID__VOID,
							 G_TYPE_NONE,
							 0);

	return;
}

static void
gem_file_chooser_dispose (GObject *object)
{
 	(*G_OBJECT_CLASS(parent_class)->dispose)(object);

 	return;
}

static void
gem_file_chooser_finalize (GObject *object)
{
	GemFileChooser * filechooser = GEM_FILE_CHOOSER(object);

	g_print("%s(): entering\n", __FUNCTION__);

	g_free(filechooser->current_path);
 //	g_object_unref(filechooser->file_system);

  (*G_OBJECT_CLASS(parent_class)->finalize)(object);
  
  return;
}

static void
gem_file_chooser_file_info_print (GtkFileInfo		*info)
{
	struct tm loacl_time;
	time_t time;
	
	if (info == NULL)
	{
		return;
	}

	time = (time_t)gtk_file_info_get_modification_time(info);
	localtime_r(&time, &loacl_time);
	
	g_print("%s():Info 0x%08x\n",
					__FUNCTION__,
					(guint32)info);
	g_print("%s():|- Display Name  : %s\n",
					__FUNCTION__,
					gtk_file_info_get_display_name(info));
	g_print("%s():|- Display Key   : %s\n",
					__FUNCTION__,
					gtk_file_info_get_display_key(info));
	g_print("%s():|- Folder        : %s\n",
					__FUNCTION__,
					(gtk_file_info_get_is_folder(info) ? "TRUE" : "FALSE"));
	g_print("%s():|- Hidden        : %s\n",
					__FUNCTION__,
					(gtk_file_info_get_is_hidden(info) ? "TRUE" : "FALSE"));
	g_print("%s():|- MIME Type     : %s\n",
					__FUNCTION__,
					gtk_file_info_get_mime_type(info));
	g_print("%s():|- Modified Time : %04d/%02d/%02d %02d:%02d:%02d\n",
					__FUNCTION__,
					loacl_time.tm_year + 1900,
					loacl_time.tm_mon + 1,
					loacl_time.tm_mday,
					loacl_time.tm_hour,
					loacl_time.tm_min,
					loacl_time.tm_sec);
	g_print("%s():`- Size          : %lld\n",
					__FUNCTION__,
					gtk_file_info_get_size(info));
					
	g_print("\n");
	
	return;
}

static void
gem_file_chooser_tree_view_row_activated (GtkTreeView				 	*treeview,
																					GtkTreePath					*path,
																					GtkTreeViewColumn		*col,
																					gpointer						 user_data)
{
	GemFileChooser * filechooser = GEM_FILE_CHOOSER(user_data);
	GtkTreeModel * model = NULL;
	GtkTreeIter iter;
	
	model = gem_tree_view_get_model(GEM_TREE_VIEW(filechooser));
	
	if (model && gtk_tree_model_get_iter(model,
																			 &iter,
																			 path))
	{
		//GtkFilePath * filepath = NULL;
		GError * error = NULL;
		gchar * name = NULL;
		gboolean is_folder = FALSE;
		
		gtk_tree_model_get(model,
											 &iter,
											 GEM_FC_COL_NAME,
											 &name,
											 GEM_FC_COL_IS_FOLDER,
											 &is_folder,
											 -1);
											 
		g_print("%s(): %-20s [%s]\n",
						__FUNCTION__,
						name ? name : "null",
						(is_folder ? "FOLER" : "FILE"));

		if (name == NULL)
		{
			return;
		}
		
		/*
		filepath = gtk_file_system_filename_to_path(filechooser->file_system,
																								filechooser->current_path);
		*/
		DBG("AAA");
    if (g_utf8_collate(name, "..") == 0)
		{
			/*GtkFilePath *parent_path = NULL;
			const gchar *parent_str = NULL;
			
			if (gtk_file_system_get_parent(filechooser->file_system,
																		 filepath,
																		 &parent_path,
																		 &error) == FALSE)
			{
				g_print("%s(): get parent error: %s\n",
								__FUNCTION__,
								error->message);
									
				g_error_free(error);
				
				return;
			}
			*/
			/*parent_str = gtk_file_path_get_string(parent_path);*/
			DBG("curr_path=%s",filechooser->current_path);
      gchar *temp = g_strdup(filechooser->current_path);
      gchar *t = g_strrstr(temp,"/");
      if(t)
        *t='\0';
      DBG("temp=%s",temp);
      gchar *parent_str=g_strdup(temp);
      DBG("pat=%s",parent_str);
      if(temp)
         g_free(temp);
      
			g_print("%s(): change to parent %s\n",
							__FUNCTION__,
							parent_str ? parent_str : "null");

			gem_file_chooser_set_current_path(filechooser,
																				parent_str);

		}
		else/*not ..*/
		{
			gchar * abosulte_path = NULL;
			abosulte_path = g_build_filename(filechooser->current_path,
																			 name,
																			 NULL);
			
			if (is_folder)
			{
				gem_file_chooser_set_current_path(filechooser,abosulte_path);
			}
			else/*is_file*/
			{
				if (filechooser->selectable)
				{
					GtkTreeIter iter;
					
					if (gtk_tree_model_get_iter(model,
																			&iter,
																			path) == TRUE)
					{
						GdkPixbuf * pixbuf = NULL;
						gchar * picpath = NULL;
						gboolean selected = FALSE;
						
						gtk_tree_model_get(model,
															 &iter,
															 GEM_FC_COL_CHECK,
															 &selected,
															 -1);
					
						selected = !selected;
						
						if (selected)
						{
							picpath = g_build_filename(gem_env_get_resource_path(),
																				 "pixmaps",
																				 "file_selected.png",
																				 NULL);
						}
						else
						{
							picpath = g_build_filename(gem_env_get_resource_path(),
																				 "pixmaps",
																				 "file.png",
																				 NULL);
						}

						if (gem_resource_load_pic(picpath,
																			GEM_PIC_NORMAL) == TRUE)
						{
							pixbuf = gem_resource_find_pic(picpath,
																						 GEM_PIC_NORMAL);
						}
						
						gtk_list_store_set(GTK_LIST_STORE(model),
															 &iter,
															 GEM_FC_COL_CHECK,
															 selected,
															 GEM_FC_COL_PIC,
															 pixbuf,
															 -1);
												
						g_free(picpath);
					}
				}
				else
				{
					g_signal_emit(G_OBJECT(filechooser),
												file_chooser_signals[FILE_SELECTED],
												0);
				}
			}

			g_free(abosulte_path);
		}

		//gtk_file_path_free(filepath);
		
		g_free(name);
	}
	
	return;
}

static gint 
gemgem_file_chooser_sort_func (GtkTreeModel 		*model,
															 GtkTreeIter			*a,
															 GtkTreeIter			*b,
															 gpointer					 user_data)
{
	gint ret = 0;
	gchar * str1 = NULL;
	gchar * str2 = NULL;
	gboolean is_folder1 = FALSE;
	gboolean is_folder2 = FALSE;
	
	gtk_tree_model_get(model,
										 a,
										 GEM_FC_COL_NAME,
										 &str1,
										 GEM_FC_COL_IS_FOLDER,
										 &is_folder1,
										 -1);
	gtk_tree_model_get(model,
										 b,
										 GEM_FC_COL_NAME,
										 &str2,
										 GEM_FC_COL_IS_FOLDER,
										 &is_folder2,
										 -1);
	
	if (str1 == NULL && str2 == NULL)
	{
		ret = 0;
	}
	else if (str1 == NULL || str2 == NULL)
	{
		ret = (str1 == NULL) ? -1 : 1;
	}
	else
	{
		if (is_folder1 != is_folder2)
		{
			ret = (is_folder1 > is_folder2) ? -1 : 1;
		}
		else
		{
			ret = g_utf8_collate(str1, str2);
		}
	}
	
	g_free(str1);
	g_free(str2);
	
	return ret;
}

static void
gem_file_chooser_file_selected (GemFileChooser	*filechooser)
{
	return;
}

static void
gem_file_chooser_init (GemFileChooser * filechooser)
{
	GtkWidget * treeview = NULL;
	GtkListStore * liststore = NULL;
	GtkCellRenderer * renderer = NULL;
	GtkTreeViewColumn	* col = NULL;

	liststore = gtk_list_store_new(GEM_FC_NUMS_COLS,
																 G_TYPE_BOOLEAN,
																 GDK_TYPE_PIXBUF,
																 G_TYPE_STRING,
																 G_TYPE_BOOLEAN);
	
	treeview = gem_tree_view_get_view(GEM_TREE_VIEW(filechooser));
		
	/* column pic */
	col = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(col, "Picutre");
	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), col);
		
	renderer = gtk_cell_renderer_pixbuf_new();
	gtk_cell_renderer_set_fixed_size(renderer, 32, 32);
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col,
																		 renderer,
																		 "pixbuf",
																		 GEM_FC_COL_PIC);
	
	/* column name */
	col = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(col, "Name");
	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), col);
		
	renderer = gtk_cell_renderer_text_new();
	gtk_cell_renderer_set_fixed_size(renderer, 130, -1);
	g_object_set(G_OBJECT(renderer), "ypad", 5, NULL);
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col,
																		 renderer,
																		 "text",
																		 GEM_FC_COL_NAME);
	
	/* set model */	
	gem_tree_view_set_model(GEM_TREE_VIEW(filechooser),
												  GTK_TREE_MODEL(liststore));
	
	gem_tree_view_set_search_column(GEM_TREE_VIEW(filechooser),
																	GEM_FC_COL_NAME,
																	GTK_SORT_ASCENDING,
																	gemgem_file_chooser_sort_func);

	g_signal_connect(G_OBJECT(treeview),
									 "row-activated",
									 G_CALLBACK(gem_file_chooser_tree_view_row_activated),
									 (gpointer)filechooser);

	gem_file_chooser_set_selectable(filechooser,
																	FALSE);
  //This should be modify to virtual root																			 
	DBG("Set root");
  gem_file_chooser_set_current_path(filechooser,
																		"/");
	DBG("Set root succesfully");
	return;
}

static void
gem_file_chooser_set_selection_all (GemFileChooser		*filechooser,
																		gboolean					 tag)
{
	GtkTreeModel * model = NULL;
	gint count = 0;
	gint i;

	if (filechooser->selectable == FALSE)
	{
		g_print("%s(): in non-selectable mode.\n",
						__FUNCTION__);
		return;
	}

	model = gem_tree_view_get_model(GEM_TREE_VIEW(filechooser));
	if (model == NULL)
	{
		return;
	}
	
	count = gtk_tree_model_iter_n_children(model, NULL);
	for (i = 1; i < count; i++)
	{
		GtkTreeIter iter;
				
		if (gtk_tree_model_iter_nth_child(model,
																			&iter,
																			NULL,
																			i) == TRUE)
		{
			gboolean selected = FALSE;
			gboolean is_folder = FALSE;
			
			gtk_tree_model_get(GTK_TREE_MODEL(model),
												 &iter,
												 GEM_FC_COL_CHECK,
												 &selected,
												 GEM_FC_COL_IS_FOLDER,
												 &is_folder,
												 -1);
			if (is_folder)
			{
				continue;
			}
												 
			if (selected != tag)
			{
				GdkPixbuf * pixbuf = NULL;
				gchar * picpath = NULL;
				
				if (tag == TRUE)
				{
					picpath = g_build_filename(gem_env_get_resource_path(),
																		 "pixmaps",
																		 "file_selected.png",
																		 NULL);
				}
				else
				{
					picpath = g_build_filename(gem_env_get_resource_path(),
																		 "pixmaps",
																		 "file.png",
																		 NULL);
				}
				
				if (gem_resource_load_pic(picpath,
																	GEM_PIC_NORMAL) == TRUE)
				{
					pixbuf = gem_resource_find_pic(picpath,
																				 GEM_PIC_NORMAL);
				}
						
				gtk_list_store_set(GTK_LIST_STORE(model),
													 &iter,
													 GEM_FC_COL_CHECK,
													 tag,
													 GEM_FC_COL_PIC,
													 pixbuf,
													 -1);
												
				g_free(picpath);
			}
		}
	}
	
	return;	
}

/* Public API
 */
GtkWidget*
gem_file_chooser_new (void)
{
  
  DBG("NEW");
  gnome_vfs_init();
  DBG("init");
  gnome_vfs_mime_info_reload ();
  DBG("reload");
  return g_object_new(GEM_TYPE_FILE_CHOOSER,
  										NULL);
}

typedef enum {
	GEM_FC_ICON_FOLDER,
	GEM_FC_ICON_FILE,
	GEM_FC_ICON_PARENT,
	GEM_FC_NUMS_ICONS
}GemFileChooserIcons;

/**a
 * gem_file_chooser_set_current_path:
 * 
 * @filechooser	:	a #GemFileChooser object
 * @pathstr			:	a file path to set as the current path of
 * 								#GemFileChooser object
 **/
void
gem_file_chooser_set_current_path (GemFileChooser			*filechooser,
																	 const gchar				*pathstr)
{
  DBG("AA");
  GdkPixbuf * icons[GEM_FC_NUMS_ICONS] = {
                                           NULL,
                                           NULL,
                                           NULL,
                                          };
                  
	GtkTreeModel * model = NULL;
  if (pathstr == NULL)
	{
    DBG("pathstr null");
		return;
	}
  
  GnomeVFSURI *uri = gnome_vfs_uri_new(pathstr);
  if (!gnome_vfs_uri_exists (uri))
  {
    DBG("path does  not exist");
    return;
  }
  gnome_vfs_uri_unref (uri);
	/*	
	path = gtk_file_system_filename_to_path(filechooser->file_system,
																					pathstr);
																						
	folder = gtk_file_system_get_folder(filechooser->file_system,
																			path,
																			GTK_FILE_INFO_ALL,
																			&error);
	if (folder == NULL)
	{
		g_print("%s(): get folder error: %s\n",
						__FUNCTION__,
						error->message);
							
		g_error_free(error);
		
		return;
	}							
  */
  
	/*
	 * list all children of current path,
	 * and add them to list sotre
	 */
	#if 0
  if (gtk_file_folder_list_children(folder,
																		&children,
																		&error) == FALSE)
	{
		g_print("%s(): get children error: %s\n",
						__FUNCTION__,
						error->message);
								
		g_error_free(error);
		
		return;		
	}
	#endif 			
	model = gem_tree_view_get_model(GEM_TREE_VIEW(filechooser));
	
	if (model)
	{
		GSList * iter = NULL;
		
    gtk_list_store_clear(GTK_LIST_STORE(model));
		if (TRUE)
		{
			gchar * fname[] = {
				"folder.png",
				"file.png",
				"parent.png",
			};
			gint i;
			
			for (i = 0; i < GEM_FC_NUMS_ICONS; i++)
			{
				gchar * picpath = NULL;
				picpath = g_build_filename( gem_env_get_resource_path(),
																	  "pixmaps",
																	  fname[i],
																	  NULL );
				DBG("i=%d,picpath=%s",i,picpath);
				if (gem_resource_load_pic(picpath,
																	GEM_PIC_NORMAL) == TRUE)
				{
					icons[i] = gem_resource_find_pic(picpath,
																					 GEM_PIC_NORMAL);
				}
				
				g_free(picpath);
			}
		}

		/*
		 * we need a ".." item to act as
		 * the parent of current path, to supply 
		 * a possiblity to user to go backward
		 */
		if (TRUE)
		{
			GtkTreeIter iter;
			
			gtk_list_store_append(GTK_LIST_STORE(model),
														&iter);
																
			gtk_list_store_set(GTK_LIST_STORE(model),
												 &iter,
												 GEM_FC_COL_NAME,
												 "..",
												 GEM_FC_COL_PIC,
												 icons[GEM_FC_ICON_PARENT],
												 GEM_FC_COL_IS_FOLDER,
												 TRUE,
												 -1);
		}
    DBG(".. is added");
#if 0
		for (iter = children; iter; iter = g_slist_next(iter))
		{
			if (iter->data)
			{
				GtkFileInfo * info = NULL;
				GtkFilePath * path = (GtkFilePath *)iter->data;
				
				info = gtk_file_folder_get_info(folder,
																				path,
																				&error);
				if (info == NULL)
				{
					g_print("%s(): get info error: %s\n",
									__FUNCTION__,
									error->message);
											
						g_error_free(error);
						error = NULL;
						
						continue;
				}

//				gem_file_chooser_file_info_print(info);
				
				/*
				 * if the child is NOT hidden, we display it
				 * in the tree view and differentiate them 
				 * with different icons
				 * 
				 * TODO:	we can get MIME type of child use the
				 * 				gtk_file_info_get_mime_type(), so we
				 * 				can import more icons to differentiate
				 * 				them with thier MIME types.
				 * 
				 */
				if (gtk_file_info_get_is_hidden(info) == FALSE)
				{
					GtkTreeIter tree_iter;
					const gchar * name = NULL;
					
					name = gtk_file_info_get_display_name(info);
									
					gtk_list_store_append(GTK_LIST_STORE(model),
																&tree_iter);
															
					if (gtk_file_info_get_is_folder(info))
					{
						gtk_list_store_set(GTK_LIST_STORE(model),
															 &tree_iter,
															 GEM_FC_COL_NAME,
															 name,
															 GEM_FC_COL_PIC,
															 icons[GEM_FC_ICON_FOLDER],
															 GEM_FC_COL_IS_FOLDER,
															 TRUE,
															 -1);
					}
					else
					{
						gtk_list_store_set(GTK_LIST_STORE(model),
															 &tree_iter,
															 GEM_FC_COL_NAME,
															 name,
															 GEM_FC_COL_PIC,
															 icons[GEM_FC_ICON_FILE],
															 GEM_FC_COL_IS_FOLDER,
															 FALSE,
															 -1);
					}
				}
				
				gtk_file_info_free(info);
			}
		}
#endif     
    {  
          
      GnomeVFSDirectoryHandle *handle = NULL;
      GnomeVFSFileInfo *vfs_file_info = NULL;
	    GtkTreeIter  tree_iter;
      GnomeVFSResult result = GNOME_VFS_OK, open_dir_result;
      open_dir_result =gnome_vfs_directory_open ( &handle, pathstr,
			                                             GNOME_VFS_FILE_INFO_DEFAULT
                                   		 	           | GNOME_VFS_FILE_INFO_FOLLOW_LINKS
                                                   | GNOME_VFS_FILE_INFO_GET_MIME_TYPE );

      while (open_dir_result == GNOME_VFS_OK)
      {
        vfs_file_info = gnome_vfs_file_info_new ();
        result = gnome_vfs_directory_read_next (handle, vfs_file_info);
        if (result != GNOME_VFS_OK)
	      {
          gnome_vfs_directory_close (handle);
	        break;
	      }
        if (vfs_file_info->name != NULL && vfs_file_info->name[0] != '.')
	      {
					gtk_list_store_append(GTK_LIST_STORE(model), &tree_iter);
          
          if (vfs_file_info->type == GNOME_VFS_FILE_TYPE_DIRECTORY)
          {
					  DBG("dir=%s is added",vfs_file_info->name);
            gtk_list_store_set(GTK_LIST_STORE(model),
															 &tree_iter,
															 GEM_FC_COL_NAME,
															 vfs_file_info->name,
															 GEM_FC_COL_PIC,
															 icons[GEM_FC_ICON_FOLDER],
															 GEM_FC_COL_IS_FOLDER,
															 TRUE,
															 -1);
					}
					else
					{
					  DBG("file=%s is added",vfs_file_info->name);
						gtk_list_store_set(GTK_LIST_STORE(model),
															 &tree_iter,
															 GEM_FC_COL_NAME,
															 vfs_file_info->name,
															 GEM_FC_COL_PIC,
															 icons[GEM_FC_ICON_FILE],
															 GEM_FC_COL_IS_FOLDER,
															 FALSE,
															 -1);
          }                     
          gnome_vfs_file_info_unref(vfs_file_info);
        }
      }
      gnome_vfs_file_info_unref(vfs_file_info);
      /*if open fail*/
      if (open_dir_result != GNOME_VFS_OK)
      {
        gchar * error=NULL;
        switch (open_dir_result)
	      {
          case GNOME_VFS_ERROR_READ_ONLY:
	             error = g_strdup (_("Destination is read only."));
     	         break;
	        case GNOME_VFS_ERROR_ACCESS_DENIED:
	             error = g_strdup (_("Access denied."));
	             break;
	        case GNOME_VFS_ERROR_READ_ONLY_FILE_SYSTEM:
	             error = g_strdup (_("Read only file system."));
	             break;
	        case GNOME_VFS_ERROR_NOT_FOUND:
               DBG("NOT found");
               break;
	        case GNOME_VFS_ERROR_GENERIC:
               DBG("generic");
	             break;
	        default:
	             
               DBG("default");
               error = g_strdup_printf ("%s %s", _("Error:"), gnome_vfs_result_to_string(open_dir_result));
	        break;
        }
        DBG("error:%s",error);
        if(error)
        {
          g_print("%s",error);
          g_free (error);
        }
      }
	  }
	}
	/*
	g_slist_free(children);	
	
	gtk_file_path_free(path);
	g_object_unref(folder);
  */
	/*
	 * log the current path and update
	 * tree view cursor to the top line
	 */
	if (filechooser->current_path != NULL)
	{
    
    DBG("Before free curr_path=%s",filechooser->current_path);
		g_free(filechooser->current_path);
    DBG("after free");
	}
  DBG("SSSS");
	
	filechooser->current_path = g_strdup(pathstr);
	DBG("set curr path=%s",filechooser->current_path);
	if (TRUE)
	{
		GtkWidget * view = NULL;
		
		view = gem_tree_view_get_view(GEM_TREE_VIEW(filechooser));
		
		if (view)
		{
			GtkTreePath * tree_path = NULL;
			GtkTreeViewColumn * col = NULL;
			
			tree_path = gtk_tree_path_new_first();
			
			col = gtk_tree_view_get_column(GTK_TREE_VIEW(view),
																		 0);
															 
			gtk_tree_view_set_cursor(GTK_TREE_VIEW(view),
															 tree_path,
															 col,
															 FALSE);
			DBG("before tree path");													 
			gtk_tree_path_free(tree_path);
		}
		
	}
	DBG("return");
	return;
}

G_CONST_RETURN gchar *
gem_file_chooser_get_current_path (GemFileChooser			*filechooser)
{
	return filechooser->current_path;
}

void
gem_file_chooser_set_selectable (GemFileChooser			*filechooser,
																 gboolean						 selectable)
{
	GtkWidget * treeview = NULL;
    
  DBG("aaaa");
	if (filechooser->selectable == selectable)
	{
		DBG("IS already selectable");
    return;
	}
    
  DBG("aaaa");
	treeview = gem_tree_view_get_view(GEM_TREE_VIEW(filechooser));
  DBG("aaaa");
	
	if (treeview)
	{
		GtkTreeModel * model = NULL;

    DBG("aaaa");
		model = gem_tree_view_get_model(GEM_TREE_VIEW(filechooser));
		
    DBG("aaaa");
		if (model)
		{
			gint count = 0;
			gint i;
			
      DBG("aaaa");
			count = gtk_tree_model_iter_n_children(model, NULL);
			for (i = 1; i < count; i++)
			{
				GtkTreeIter iter;
				
				if (gtk_tree_model_iter_nth_child(model,
																					&iter,
																					NULL,
																					i) == TRUE)
				{
					GdkPixbuf * pixbuf = NULL;
					gchar * picpath = NULL;
					gboolean is_folder = FALSE;
					
					gtk_tree_model_get(model,
														 &iter,
														 GEM_FC_COL_IS_FOLDER,
														 &is_folder,
														 -1);
					if (is_folder)
					{
						continue;
					}														 
					
					picpath = g_build_filename(gem_env_get_resource_path(),
																		 "pixmaps",
																		 "file.png",
																		 NULL);
          DBG("picpath=%s",picpath);
					if (gem_resource_load_pic(picpath,
																		GEM_PIC_NORMAL) == TRUE)
					{
						pixbuf = gem_resource_find_pic(picpath,
																					 GEM_PIC_NORMAL);
					}
					
					gtk_list_store_set(GTK_LIST_STORE(model),
														 &iter,
														 GEM_FC_COL_PIC,
														 pixbuf,
														 GEM_FC_COL_CHECK,
														 FALSE,
														 -1);
					
					g_free(picpath);
				}
			}
		}
		
		if (FALSE)
		{
			GtkTreePath * tree_path = NULL;
			
			tree_path = gtk_tree_path_new_first();
			
			gtk_tree_view_set_cursor(GTK_TREE_VIEW(treeview),
															 tree_path,
															 NULL,
															 FALSE);
																 
			gtk_tree_path_free(tree_path);
		}
	}
	
	filechooser->selectable = selectable;

//	g_object_unref(treeview);										 
	DBG("return");
	return;
}

gboolean
gem_file_chooser_get_selectable (GemFileChooser			*filechooser)
{
	return filechooser->selectable;
}

GList *
gem_file_chooser_get_current_selections (GemFileChooser		*filechooser)
{
	GtkTreeModel * model = NULL;
	GList * results = NULL;
	gint count = 0;
	gint i;

	model = gem_tree_view_get_model(GEM_TREE_VIEW(filechooser));
	if (model == NULL)
	{
		return results;
	}
	
	if (filechooser->selectable == FALSE)
	{
		GtkWidget * treeview = NULL;

		treeview = gem_tree_view_get_view(GEM_TREE_VIEW(filechooser));
		if (treeview)
		{
			GtkTreePath * path = NULL;
			gtk_tree_view_get_cursor(GTK_TREE_VIEW(treeview),
															 &path,
															 NULL);
			if (path)
			{
				GtkTreeIter iter;
				
				if (gtk_tree_model_get_iter(model,
																		&iter,
																		path) == TRUE)
				{
					gchar * fname = NULL;
					
					gtk_tree_model_get(GTK_TREE_MODEL(model),
														 &iter,
														 GEM_FC_COL_NAME,
														 &fname,
														 -1);
				
					results = g_list_append(results,
																	(gpointer)fname);
					
				}
				
				gtk_tree_path_free(path);
			}
		}
		
		return results;
	}

	count = gtk_tree_model_iter_n_children(model, NULL);
	for (i = 1; i < count; i++)
	{
		GtkTreeIter iter;
				
		if (gtk_tree_model_iter_nth_child(model,
																			&iter,
																			NULL,
																			i) == TRUE)
		{
			gchar * fname = NULL;
			gboolean selected = FALSE;
			
			gtk_tree_model_get(GTK_TREE_MODEL(model),
												 &iter,
												 GEM_FC_COL_NAME,
												 &fname,
												 GEM_FC_COL_CHECK,
												 &selected,
												 -1);
												 
			if(selected == TRUE)
			{
/*				gchar * abosulte_path = NULL;
				
				abosulte_path = g_build_filename(filechooser->current_path,
																				 fname,
																				 NULL);
*/				
				results = g_list_append(results,
																(gpointer)fname);
			}
			
//			g_free(fname);
		}
	}
	
	return results;	
}

void
gem_file_chooser_select_all (GemFileChooser		*filechooser)
{
	return gem_file_chooser_set_selection_all(filechooser, TRUE);
}

void
gem_file_chooser_unselect_all (GemFileChooser		*filechooser)
{
	return gem_file_chooser_set_selection_all(filechooser, FALSE);
}

/*
vi:ts=2:nowrap:ai:expandtab 
*/
